use std::{collections::VecDeque, env, fs::File, io::Read};

mod client;
mod server;
mod voice;

#[derive(Debug, Clone)]
struct TOML {
    header: String,
    fields: VecDeque<String>,
}

const SERVER_CONFIG_FILE_LOCATION: &str = "configs/server_config.toml";
const CLIENT_CONFIG_FILE_LOCATION: &str = "configs/client_config.toml";
const BUFFER_LENGTH: usize = 1000000;

#[derive(Debug)]
struct ServerConfig<'a> {
    server_address: &'a str,
}

#[derive(Debug)]
struct ClientConfig<'a> {
    server_address: &'a str,
}

async fn server() {
    let mut config = naive_toml_parser(SERVER_CONFIG_FILE_LOCATION);

    if config.header.as_str() == "server_config" {
        let server_config = ServerConfig {
            server_address: &config.fields.pop_front().unwrap(),
        };
        server::run(server_config).await;
    } else {
        eprintln!("Error: Server Config File Couldn't Be Parsed Correctly");
    }
}

async fn client() {
    let mut config = naive_toml_parser(CLIENT_CONFIG_FILE_LOCATION);

    if config.header.as_str() == "client_config" {
        let client_config = ClientConfig {
            server_address: &config.fields.pop_front().unwrap(),
        };
        client::run(client_config).await;
    } else {
        eprintln!("Error: Client Config File Couldn't Be Parsed Correctly");
    }
}

pub async fn start() {
    match env::args().collect::<Vec<String>>().get(1) {
        Some(arg) => match arg.as_str() {
            "-c" | "--client" => client().await,
            "-s" | "--server" => server().await,
            _ => return,
        },
        None => return,
    }
}

fn naive_toml_parser(file_location: &str) -> TOML {
    let mut toml_file = File::open(file_location).unwrap();
    let mut toml_ingredients = String::default();
    toml_file.read_to_string(&mut toml_ingredients).unwrap();
    let mut toml_ingredients = toml_ingredients.lines().collect::<VecDeque<&str>>();

    let header = toml_ingredients
        .pop_front()
        .unwrap()
        .replace('[', "")
        .replace(']', "")
        .trim_end()
        .to_string();

    let fields = toml_ingredients
        .iter()
        .map(|ingredient| {
            ingredient
                .split_once('=')
                .unwrap()
                .1
                .replace('"', "")
                .trim()
                .to_string()
        })
        .collect();

    TOML { header, fields }
}
