use std::{io, path::Path};

use s2n_quic::Server;
use tokio::{
    io::AsyncReadExt,
    sync::{broadcast, oneshot},
};

use crate::{
    BUFFER_LENGTH, ServerConfig,
    voice::{play, record},
};

pub async fn run<'a>(server_config: ServerConfig<'a>) {
    let mut server = Server::builder()
        .with_io(server_config.server_address)
        .unwrap()
        .with_tls((
            Path::new("certificates/cert.pem"),
            Path::new("certificates/key.pem"),
        ))
        .unwrap()
        .start()
        .unwrap();
    while let Some(mut connection) = server.accept().await {
        println!(
            "Server Name = {}",
            connection.server_name().unwrap().unwrap().to_string()
        );

        println!(
            "Client Address = {}",
            connection.remote_addr().unwrap().to_string()
        );

        let stream = connection
            .accept_bidirectional_stream()
            .await
            .unwrap()
            .unwrap();
        let (mut receive_stream, mut send_stream) = stream.split();

        let (microphone_sender, mut microphone_receiver) = broadcast::channel::<f32>(BUFFER_LENGTH);
        let (speaker_sender, speaker_receiver) = broadcast::channel::<f32>(BUFFER_LENGTH);

        let (microphone_stop_signal_sender, microphone_stop_signal_receiver) =
            oneshot::channel::<bool>();
        let (spearker_stop_signal_sender, speaker_stop_signal_receiver) =
            oneshot::channel::<bool>();

        tokio::spawn(play(speaker_receiver, speaker_stop_signal_receiver));
        tokio::spawn(record(microphone_sender, microphone_stop_signal_receiver));
        tokio::spawn(async move {
            while let Ok(data) = receive_stream.read_f32_le().await {
                speaker_sender.send(data).unwrap();
            }
        });
        tokio::spawn(async move {
            while let Ok(data) = microphone_receiver.recv().await {
                send_stream
                    .send(data.to_le_bytes().to_vec().into())
                    .await
                    .unwrap();
            }
        });
        let mut read_buffer = String::default();
        io::stdin().read_line(&mut read_buffer).unwrap();
        microphone_stop_signal_sender.send(true).unwrap();
        spearker_stop_signal_sender.send(true).unwrap();
    }
}
