use cpal::traits::{DeviceTrait, HostTrait, StreamTrait};
use tokio::sync::{broadcast, oneshot};

pub async fn record(
    microphone_sender: broadcast::Sender<f32>,
    stop_signal_receiver: oneshot::Receiver<bool>,
) {
    let host = cpal::default_host();
    let input_device = host.default_input_device().unwrap();
    let config = input_device.default_input_config().unwrap().into();

    let input = move |data: &[f32], _: &cpal::InputCallbackInfo| {
        for &sample in data {
            if microphone_sender.receiver_count() > 0 {
                if let Err(err_val) = microphone_sender.send(sample) {
                    eprintln!("Error: Microphone Send | {}", err_val);
                }
            }
        }
    };

    let input_stream = input_device
        .build_input_stream(&config, input, voice_error, None)
        .unwrap();
    input_stream.play().unwrap();
    println!("Recording Started");

    tokio::task::block_in_place(|| {
        let _ = stop_signal_receiver.blocking_recv();
    });

    input_stream.pause().unwrap();
}

pub async fn play(
    mut speaker_receiver: broadcast::Receiver<f32>,
    stop_signal_receiver: oneshot::Receiver<bool>,
) {
    let host = cpal::default_host();
    let output_device = host.default_output_device().unwrap();
    let config = output_device.default_output_config().unwrap().into();

    let output = move |data: &mut [f32], _: &cpal::OutputCallbackInfo| {
        for sample in data {
            if speaker_receiver.len() > 0 {
                match speaker_receiver.blocking_recv() {
                    Ok(received_sample) => *sample = received_sample,
                    Err(err_val) => eprintln!("Error: Speaker Receive | {}", err_val),
                }
            } else {
                *sample = 0.0;
            }
        }
    };

    let output_stream = output_device
        .build_output_stream(&config, output, voice_error, None)
        .unwrap();

    output_stream.play().unwrap();
    println!("Playing Started");

    tokio::task::block_in_place(|| {
        let _ = stop_signal_receiver.blocking_recv();
    });

    output_stream.pause().unwrap();
}

fn voice_error(err_val: cpal::StreamError) {
    eprintln!("Error: Voice Error | {}", err_val);
}
